# frozen_string_literal: true
# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class EntitiesController < Hermes::Api::Dmo::ApplicationController

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          ids
          als_id
          title
          siret
          siren
          is_root
          root_id
          parent_id
          id_rex
          id_cristal
          automated_request_validations
          delegated_validations
          priorities_enabled
          in_peec
          achieved_goal
          spent_on_completed_requests
          spent_on_ongoing_requests
          spent_on_other
          annual_stock
          total_spent
          available_points
          applicant_ids
          housing_offer_ids
          lessor_id
          group_id
          supervisor_group_id
          manager_group_id
          address
          postal_code
          district
          country
          latitude
          longitude
          auto_validation
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          title
          siret
          siren
          parent_id
          id_rex
          id_cristal
          automated_request_validations
          delegated_validations
          priorities_enabled
          in_peec
          achieved_goal
          spent_on_completed_requests
          spent_on_ongoing_requests
          spent_on_other
          annual_stock
          applicant_ids
          housing_offer_ids
          lessor_id
          address
          postal_code
          district
          country
          latitude
          longitude
          auto_validation
        ].freeze
        mutate_params_only(*MUTATE_PARAMS)

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
          @query_precision ||= {}
          if params[:housing_offer_ids]
            housing_entity_ids = Hermes::HousingEntityOffer.only(:id, :housing_offer_id)
                                                           .where(
                                                             housing_offer_id: params[:housing_offer_ids]
                                                           ).pluck(:id)
            @query_precision[:housing_entity_offer_ids] = housing_entity_ids
          end
          super
        end
          
        def is_super_admin
          current_user_roles.include?('Superadmin')
        end

        def admin_search
          if is_super_admin
            if !params['entity_params'].nil?
              
              params_entity = JSON.parse(params['entity_params'])

              entity_query = {}

              unless (id = params_entity[:id]).blank?
                entity_query[:id] = id
              end

              unless (id_cristal = params_entity[:id_cristal]).blank?
                  entity_query[:id_cristal] = id_cristal
              end
        
              unless (siren = params_entity[:siren]).blank?
                  entity_query[:siren] = siren
              end
        
              unless (siret = params_entity[:siret]).blank?
                entity_query[:siret] = siret
              end

              unless (title = params_entity[:title]).blank?
                entity_query[:title] = title
              end
        
  
        
              entity_list = Hermes::Entity.where(entity_query).to_a


              if !params['user_params'].nil?
                user_params = JSON.parse(params['user_params'])
                
                new_list = []
                unless entity_list.blank?
                  entity_list.each do |entity|
                    users = []
                    entity_response = {
                      id: entity.id,
                      id_cristal: entity.id_cristal,
                      siren: entity.siren,
                      siret: entity.siret,
                      title: entity.title
                    }

                    users_found_by_id = []

                    entity.all_users.each do |uid|
                        user = User.find(uid)
                        users_found_by_id << user
                    end   
                    
                    user_params = JSON.parse(params['user_params'])

                    user_query = {}
                    unless (id = user_params[:id]).blank?
                        user_query[:id] = id
                    end
              
                    unless (first_name = user_params[:first_name]).blank?
                        user_query[:first_name] = first_name
                    end
              
                    unless (last_name = user_params[:last_name]).blank?
                      user_query[:last_name] = last_name
                    end

                    unless (auth_roles = user_params[:auth_roles]).blank?
                      user_query[:auth_roles] = auth_roles
                    end
              
        
              
                    user_list = User.where(user_query).to_a
      
      
                    
                    users_final = []
                    unless user_list.blank?
                      users_found_by_id.each do |user_found_by_id|
                          user_list.each do |user_found_by_filters|
                              if user_found_by_filters == user_found_by_id
                                  users_final << user_found_by_id
                              end 
                          end 
                      end
                    end 

                    users_final.each do |user|
                        entity_user = {
                          id: user.id,
                          first_name: user.first_name,
                          last_name: user.last_name,
                          auth_roles: user.auth_roles
                        }
                        users << entity_user
                    end 

                    new_entrie = {"entity" => entity_response, "user" => users}
                    new_list << new_entrie
                  end
                end 

                render json: {status: "success", code: 200, message: {"entities_and_users" => new_list}}
              else

                new_list = []
                unless entity_list.nil?
                  entity_list.each do |entity|
                    users = []
                    entity_response = {
                      id: entity.id,
                      id_cristal: entity.id_cristal,
                      siren: entity.siren,
                      siret: entity.siret,
                      title: entity.title
                    }

                    users_found_by_id = []
                    entity.all_users.each do |uid|
                      user = User.find(uid)
                      users_found_by_id << user
                    end   

                    users_found_by_id.each do |user|
                      entity_user = {
                        id: user.id,
                        first_name: user.first_name,
                        last_name: user.last_name,
                        auth_roles: user.auth_roles
                      }
                      users << entity_user
                    end 
                    new_entrie = {"entity" => entity_response, "user" => users}
                    new_list << new_entrie
                  end 
                end 
                render json: {status: "success", code: 200, message:  {"entities_and_users" => new_list}}
              end
            else 
              render json: {status: "unauthorized", code: 401, message: "no entity params detected"}
            end 
          else 
            render json: {status: "unauthorized", code: 401, message: "user is no superadmin"}
          end  
        end
        
        def is_a_company_collaborator
          current_user_roles.include?('collaborateur offre logement')
          true
        end
      

        def add_points
          if !params[:points_to_add].nil?
            if is_a_company_collaborator
              points_to_add = params[:points_to_add].to_i
              entity_id = params[:entity_id]
              entity = Hermes::Entity.find(entity_id)
              response_points = entity.add_points(@user, points_to_add)
              render json: response_points
            else 
              render json: {status: "unauthorized", code: 401, message: 'access denied user is no collabroator'}
            end 
          else
            render json: {status: "missing param", code: 404, message: 'no points to add in param'} 
          end 
        end 

        def can_create_tier_account
          Hermes::Entity.readable_by(current_user).find_by(id: @instance.id) && 
          (
            current_user.auth_roles&.include?('entreprise') || 
            current_user.auth_roles&.include?('entreprise administrateur')
          )
        end

        def checks_before_create_account
          @instance = Hermes::Entity.find_by(id: params[:id]) if params[:id]
          return [:not_found, :instance_not_found] unless @instance
          return [:unauthorized, :no_right_on_this_entity] if !can_create_tier_account
          if [:id, :first_name, :last_name, :email, :gender, :phone_number].any? { |p| !params.has_key?(p)  }
            return [:unprocessable_entity, :user_information_missing] 
          end
        end 

        def is_superadmin
          current_user.has_role?('Superadmin')
        end 

        def switch_manual_mode
          if is_superadmin
            entity = Hermes::Entity.find(params[:id])
            if entity.switch_manual_mode
              render json: {status: "success", code: 200, message: "manual mode switch to #{entity.manual_mode}"}
            else
              render json: {status: "not found", code: 404, message: 'entity not found'}
            end 
          else 
            render json: {status: "unauthorized", code: 401, message: "user is not a superadmin"}
          end 
        end 

        def generate_user_right_data(users, type)
          users.map { |u| {user_id: u.id.to_s, email: u.email, right: type}} 
        end

        def user_rights
          return render_json_error(:not_found, :instance_not_found) unless @instance

          if params[:user_id]
            user = User.find(params[:user_id])
            return render_json_error(:not_found, :user_not_found) unless user
            
            right = nil
            
            user.group_ids.each do |id|
              right = 'reader' if id == @instance.group_id
              right = 'user' if id == @instance.supervisor_group_id
              right = 'administrator' if id == @instance.manager_group_id
            end
            
            data = {}
            data = {user_id: user.id.to_s, email: user.email, right: right} if right
          else
            data = []
  
            users = User.where(group_ids: @instance.group_id)          
            readers = User.where(group_ids: @instance.supervisor_group_id)          
            administrator = User.where(group_ids: @instance.manager_group_id)
            
            data = data + generate_user_right_data(users, 'user')
            data = data + generate_user_right_data(readers, 'reader')
            data = data + generate_user_right_data(administrator, 'administrator')
          end

          render json: data
        end

      end
    end
  end
end
